@@ -43,6 +43,9 @@
 /* Define if you want Rob Nation's own graphic mode */
 #undef RXVT_GRAPHICS
 
+/* Define if you want to use Rxvt original style scrollbars */
+#undef RXVT_SCROLLBAR
+
 /* Define if you want to use NeXT style scrollbars */
 #undef NEXT_SCROLLBAR
 
@@ -92,22 +95,36 @@
  * Use wheel events (button4 and button5) to scroll.  Even if you don't
  * have a wheeled mouse, this is harmless unless you have an exotic mouse!
  */
-#undef NO_MOUSE_WHEEL
+#undef MOUSE_WHEEL
 
 /* Define if you don't want handling for rarely used features */
 #undef NO_FRILLS
+
+/* Define to use a 24 bit visual if the screen has 24 bit mode, even if
+ * the default is 8 bit */
+#undef PREFER_24BIT
+
+/* Build shared library version - don't set here, specify via configure */
+#undef LIBRXVT
+
+/* Define to use "smart" resize behavior */
+#undef SMART_RESIZE
+
 /* ------------------------------------------------------------------------- */
 /* --                     CONFIGURE DETECTED FEATURES                     -- */
 /* ------------------------------------------------------------------------- */
 @TOP@
-/* Define if Xlocale support doesn't work */
-#undef NO_XLOCALE
+/* Define if Xlocale support works */
+#undef HAVE_XLOCALE
 
-/* Define if setlocale (defined to Xsetlocale) doesn't work */
-#undef NO_XSETLOCALE
+/* Define if setlocale (defined to Xsetlocale) works */
+#undef HAVE_XSETLOCALE
  
-/* Define if plain old setlocale doesn't work */
-#undef NO_SETLOCALE
+/* Define if plain old setlocale works */
+#undef HAVE_SETLOCALE
+
+/* Define if nl_langinfo(CODESET) works */
+#undef HAVE_NL_LANGINFO
 
 /* Define if utmp.h has struct utmp */
 #undef HAVE_STRUCT_UTMP
