@@ -1,6 +1,6 @@
 /*
  * File:	feature.h
- * $Id: feature.h,v 1.38 2000/01/14 01:26:24 mason Exp $
+ * $Id: feature.h,v 1.59 2001/04/04 11:34:18 gcw Exp $
  *
  * Compile-time configuration.
  *-----------------------------------------------------------------------
@@ -46,9 +46,14 @@
 /* #define NO_BRIGHTCOLOR */
 
 /*
- * Disable separate colours for bold/underline
+ * Make colours match xterm colours instead of `traditional' rxvt colours
  */
-/* #define NO_BOLDUNDERLINE */
+#define XTERM_COLORS
+
+/*
+ * Disable separate colours for bold, underline and reverse video
+ */
+/* #define NO_BOLD_UNDERLINE_REVERSE */
 
 /*
  * Disable using simulated bold using overstrike.  You can also turn off
@@ -69,9 +74,10 @@
 /* #define NO_BOLDFONT */
 
 /*
- * If the screen has 24 bit mode, use that even if the default is 8 bit.
+ * Define maximum possible columns and rows
  */
-#define PREFER_24BIT
+#define MAX_COLS	400
+#define MAX_ROWS	300
 
 /*
  * Define default colours for certain items.  If you have a low colour
@@ -108,10 +114,10 @@
 #define COLOR_CURSOR_BACKGROUND	NULL	/* if NULL, use foreground colour */
 
 /*
- * Use alternative code for screen-refreshes when compiled with xpm-support.
- * Seems to be obsolete due to the new screen-update routines.
+ * Define to remove support for XCopyArea() support.  XCopyArea() is useful
+ * for scrolling on non-local X displays
  */
-/* #define XPM_BUFFERING */
+/* #define NO_SLOW_LINK_SUPPORT */
 
 /*
  * Printer pipe which will be used for emulation of attached vt100 printer
@@ -121,16 +127,20 @@
 /*
  * Allow 80/132 mode switching on startup
  */
-/* define ALLOW_132_MODE */
+/* #define ALLOW_132_MODE */
  
 /*------------------------------RESOURCES-------------------------------*/
 /*
- * Define where to find installed application defaults for rxvt
+ * Define to find installed application defaults for rxvt
  * Only if USE_XGETDEFAULT is not defined.
  */
 /* #define USE_XAPPLOADDIR */
+/*
+ * Paths for installed application defaults for rxvt
+ */
 #ifndef XAPPLOADDIR
-# define XAPPLOADDIR	"/usr/lib/X11/app-defaults"
+# define XAPPLOADDIR		"/usr/X11R6/lib/X11/app-defaults" 
+# define XAPPLOADDIRLOCALE	"/usr/X11R6/lib/X11/%-.50s/app-defaults"
 #endif
 
 /*
@@ -183,10 +193,11 @@
 /*
  * Allow scrolling with modifier+Up/Down keys, in addition
  * to modifier+Prior/Next? (modifier is controlled with
- * SCROLL_ON_* defines above.)
- *
+ * SCROLL_ON_* defines above.). 
+ * Also for modifier+Home/End keys to move to top/bottom
  */
 /* #define SCROLL_ON_UPDOWN_KEYS */
+/* #define SCROLL_ON_HOMEEND_KEYS */
 
 /*
  * Allow unshifted Next/Prior keys to scroll forward/back
@@ -200,6 +211,13 @@
  */
 #define PAGING_CONTEXT_LINES 1 /* */
 
+/*
+ * Have either Ctrl+Tab or Mod4+Tab emit \e\t
+ * Useful when window manager grabs Alt+Tab   -- mg
+ */
+/* #define CTRL_TAB_MAKES_META */
+/* #define MOD4_TAB_MAKES_META */
+
 /*--------------------------------MOUSE---------------------------------*/
 /*
  * Disable sending escape sequences (up, down, page up/down)
@@ -224,10 +242,16 @@
 /* #define MOUSE_REPORT_DOUBLECLICK */
 
 /*
- * Set delay between multiple click events [default: 500]
+ * Set delay between multiple click events [default: 500 milliseconds]
  */
 /* #define MULTICLICK_TIME 500 */
 
+/*
+ * Time factor to slow down a `jumpy' mouse.  Motion isn't recognised until
+ * this long after the last mouse click [default: 50 milliseconds]
+ */
+#define MOUSE_THRESHOLD		50
+
 /*
  * If mouse wheel is defined, then scrolling is by 5 lines (or 1 line
  * if the shift key is down).  Scrolling can be smooth or jump scrolling
@@ -255,7 +279,7 @@
 /*
  * Disable all bell indications
  */
-/* define NO_BELL */
+/* #define NO_BELL */
 
 /*
  * Disable automatic de-iconify when a bell is received
@@ -269,11 +293,10 @@
 
 /*-----------------------------SCROLL BAR-------------------------------*/
 /*
- * Choose the scrollbar width - should be an even number [default: 10]
- * Except for XTERM_SCROLLBAR: it is *always* 15
- * 	and for NEXT_SCROLLBAR, which is *always* 19
+ * Choose the rxvt style scrollbar width
+ * - should be an even number [default: 10]
  */
-/* #define SB_WIDTH 10 */
+/* #define SB_WIDTH_RXVT 10 */
 
 /*
  * When using Rxvt scrollbar, clicking above or below the slider will move
@@ -349,6 +372,11 @@
  */
 /* #define ESCZ_ANSWER	"\033[?1;2C" */
 
+/*
+ * Don't bother cleaning up pixel droppings
+ */
+/* #define NO_PIXEL_DROPPING_AVOIDANCE */
+
 /*
  * Check the current value of the window-time/icon-name and avoid
  * re-setting it to the same value -- avoids unnecessary window refreshes
@@ -366,6 +394,12 @@
  */
 /* #define NO_SECONDARY_SCREEN_CURSOR */
 
+/*
+ * If no secondary screen is available (NO_SECONDARY_SCREEN in config.h),
+ * then scroll the text up on a window change request
+ */
+/* #define SCROLL_ON_NO_SECONDARY */
+
 /*
  * Width of the term internal border
  */
@@ -382,24 +416,13 @@
 #define SAVELINES	64
 
 /*
- * Provide support for pathetic applications which expect specifically
- * undefined "bw" (termcap/terminfo) behaviour to be specifically defined
- * as xterm defines it.
+ * Provide termcap/terminfo bw support
  */
-#define SUPPORT_BROKEN_APPS_WHICH_RELY_ON_UNDEFINED_BW_BEHAVIOUR_AS_XTERM
+/* #define TERMCAP_HAS_BW */
 
 /*
- * List of default fonts available
- * NFONTS is the number of fonts in the list
- * FONT0_IDX is the default font in the list (starting at 0)
- * Sizes between multi-char fonts sets (MFONT_LIST) and single-char font
- * sets (NFONT_LIST) have been matched up
+ * MAX_NFONTS is the maximum allowed number of fonts in the list
  */
-#define NFONTS		7
-#define FONT0_IDX	2
-#define NFONT_LIST	"7x14", "6x10", "6x13", "8x13", "8x16", "10x20", "12x24"
-#ifdef MULTICHAR_SET		/* multichar glyph language support */
-# define MFONT_LIST	NULL, NULL,  NULL, NULL, NULL, NULL, NULL
-#endif
+#define MAX_NFONTS	7
 
 #endif
