@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	graphics.c
  *----------------------------------------------------------------------*
- * $Id: graphics.c,v 1.24 1999/12/18 06:51:23 mason Exp $
+ * $Id: graphics.c,v 1.37 2001/04/05 10:41:36 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1994      Rob Nation <nation@rocket.sanders.lockheed.com>
@@ -27,6 +27,16 @@
 #include "../config.h"		/* NECESSARY */
 #ifdef RXVT_GRAPHICS
 #include "rxvt.h"		/* NECESSARY */
+#include "rxvtgrx.h"
+
+typedef struct grcmd_t {
+    char            cmd;
+    short           color;
+    short           ncoords;
+    int            *coords;
+    unsigned char  *text;
+    struct grcmd_t *next;
+} grcmd_t;
 
 typedef struct grwin_t {
     Window          win;
@@ -54,16 +64,13 @@ typedef struct grwin_t {
 # define GRX_SCALE	10000
 #endif
 
-static int      graphics_up = 0;
-static grwin_t *gr_root = NULL;
-
 /*----------------------------------------------------------------------*
  * local functions
  */
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_NewWindow(int nargs, int args[])
+rxvt_Gr_NewWindow(rxvt_t *r, int nargs, int args[])
 {
     int             x, y;
     unsigned int    w, h;
@@ -72,26 +79,26 @@ Gr_NewWindow(int nargs, int args[])
     Cursor          cursor;
 
     if (nargs != 4) {
-	print_error("NewWindow: 4 args needed, got %d\n", nargs);
+	rxvt_print_error("NewWindow: 4 args needed, got %d\n", nargs);
 	return;
     }
-    x = args[0] * TermWin.width / GRX_SCALE + TermWin.int_bwidth;
-    y = args[1] * TermWin.height / GRX_SCALE + TermWin.int_bwidth;
-    w = args[2] * TermWin.width / GRX_SCALE;
-    h = args[3] * TermWin.height / GRX_SCALE;
+    x = args[0] * TermWin_TotalWidth() / GRX_SCALE;
+    y = args[1] * TermWin_TotalHeight() / GRX_SCALE;
+    w = args[2] * TermWin_TotalWidth() / GRX_SCALE;
+    h = args[3] * TermWin_TotalHeight() / GRX_SCALE;
 
-    win = XCreateSimpleWindow(Xdisplay, TermWin.vt,
+    win = XCreateSimpleWindow(r->Xdisplay, r->TermWin.vt,
 			      x, y, w, h,
 			      0,
-			      PixColors[Color_fg],
-			      PixColors[Color_bg]);
+			      r->PixColors[Color_fg],
+			      r->PixColors[Color_bg]);
 
-    cursor = XCreateFontCursor(Xdisplay, XC_crosshair);
-    XDefineCursor(Xdisplay, win, cursor);
-    XMapWindow(Xdisplay, win);
-    XSelectInput(Xdisplay, win, ExposureMask);
+    cursor = XCreateFontCursor(r->Xdisplay, XC_crosshair);
+    XDefineCursor(r->Xdisplay, win, cursor);
+    XMapWindow(r->Xdisplay, win);
+    XSelectInput(r->Xdisplay, win, ExposureMask);
 
-    grwin = (grwin_t *) MALLOC(sizeof(grwin_t));
+    grwin = (grwin_t *) malloc(sizeof(grwin_t));
     grwin->win = win;
     grwin->x = x;
     grwin->y = y;
@@ -99,20 +106,20 @@ Gr_NewWindow(int nargs, int args[])
     grwin->h = h;
     grwin->screen = 0;
     grwin->prev = NULL;
-    grwin->next = gr_root;
+    grwin->next = r->h->gr_root;
     if (grwin->next)
 	grwin->next->prev = grwin;
-    gr_root = grwin;
+    r->h->gr_root = grwin;
     grwin->graphics = NULL;
-    graphics_up++;
+    r->h->graphics_up++;
 
-    tt_printf("\033W%ld\n", (long)grwin->win);
+    rxvt_tt_printf(r, "\033W%ld\n", (long)grwin->win);
 }
 
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_ClearWindow(grwin_t *grwin)
+rxvt_Gr_ClearWindow(rxvt_t *r, grwin_t *grwin)
 {
     grcmd_t        *cmd, *next;
 
@@ -124,7 +131,7 @@ Gr_ClearWindow(grwin_t *grwin)
 	free(cmd);
     }
     grwin->graphics = NULL;
-    XClearWindow(Xdisplay, grwin->win);
+    XClearWindow(r->Xdisplay, grwin->win);
 }
 
 /*
@@ -136,7 +143,7 @@ Gr_ClearWindow(grwin_t *grwin)
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_Text(grwin_t *grwin, grcmd_t *data)
+rxvt_Gr_Text(rxvt_t *r, grwin_t *grwin, grcmd_t *data)
 {
     int             x, y, align;
 
@@ -147,59 +154,55 @@ Gr_Text(grwin_t *grwin, grcmd_t *data)
     y = data->coords[1] * grwin->h / GRX_SCALE;
     align = data->coords[2];
 
-    if (align & RIGHT_TEXT)
-	x -= XTextWidth(TermWin.font, data->text, data->coords[3]);
-    else if (align & HCENTER_TEXT)
-	x -= (XTextWidth(TermWin.font, data->text, data->coords[3]) >> 1);
+    if ((align & HORIZONTAL_ALIGNMENT) == RIGHT_TEXT)
+	x -= XTextWidth(r->TermWin.font, data->text, data->coords[3]);
+    else if ((align & HORIZONTAL_ALIGNMENT) == HCENTER_TEXT)
+	x -= (XTextWidth(r->TermWin.font, data->text, data->coords[3]) >> 1);
 
-    if (align & TOP_TEXT)
-	y += TermWin.font->ascent;
-    else if (align & BOTTOM_TEXT)
-	y -= TermWin.font->descent;
+    if ((align & VERTICAL_ALIGNMENT) == TOP_TEXT)
+	y += r->TermWin.font->ascent;
+    else if ((align & VERTICAL_ALIGNMENT) == BOTTOM_TEXT)
+	y -= r->TermWin.font->descent;
 
-    if (align & VCENTER_TEXT)
-	y -= TermWin.font->descent
-	     + ((TermWin.font->ascent + TermWin.font->descent) >> 1);
-    if (align & VCAPS_CENTER_TEXT)
-	y += (TermWin.font->ascent >> 1);
+    if ((align & VERTICAL_ALIGNMENT) == VCENTER_TEXT)
+	y -= r->TermWin.font->descent
+	     + ((r->TermWin.font->ascent + r->TermWin.font->descent) >> 1);
+    if ((align & VERTICAL_ALIGNMENT) == VCAPS_CENTER_TEXT)
+	y += (r->TermWin.font->ascent >> 1);
 
-    XPMClearArea(Xdisplay, grwin->win, x, y, Width2Pixel(data->coords[3]),
-		 Height2Pixel(1), 0);
-    XDrawString(Xdisplay, grwin->win, TermWin.gc, x, y,
+    XPMClearArea(r->Xdisplay, grwin->win, x, y - r->TermWin.font->ascent,
+		 Width2Pixel(data->coords[3]), Height2Pixel(1), 0);
+    XDrawString(r->Xdisplay, grwin->win, r->TermWin.gc, x, y,
 		data->text, data->coords[3]);
 }
 
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_Geometry(grwin_t *grwin, grcmd_t *data)
+rxvt_Gr_Geometry(rxvt_t *r, grwin_t *grwin, grcmd_t *data __attribute__((unused)))
 {
     if (grwin)
-	tt_printf("\033G%ld %d %d %u %u %d %d %ld %ld %d\n",
+	rxvt_tt_printf(r, "\033G%ld %d %d %u %u %d %d %ld %ld %d\n",
 		  (long)grwin->win,
 		  grwin->x, grwin->y, grwin->w, grwin->h,
-		  TermWin.fwidth,
-		  TermWin.fheight,
-		  (long)GRX_SCALE * TermWin.fwidth / grwin->w,
-		  (long)GRX_SCALE * TermWin.fheight / grwin->h,
-		  Xdepth);
+		  r->TermWin.fwidth,
+		  r->TermWin.fheight,
+		  (long)GRX_SCALE * r->TermWin.fwidth / grwin->w,
+		  (long)GRX_SCALE * r->TermWin.fheight / grwin->h,
+		  XDEPTH);
     else			/* rxvt terminal window size */
-	tt_printf("\033G0 0 0 %d %d %d %d %ld %ld %d\n",
-		  TermWin.width - 2 * TermWin.int_bwidth,
-		  TermWin.height - 2 * TermWin.int_bwidth,
-		  TermWin.fwidth,
-		  TermWin.fheight,
-		  (long)GRX_SCALE * TermWin.fwidth
-		        / (TermWin.width - 2 * TermWin.int_bwidth),
-		  (long)GRX_SCALE * TermWin.fheight
-		        / (TermWin.height - 2 * TermWin.int_bwidth),
-		  Xdepth);
+	rxvt_tt_printf(r, "\033G0 0 0 %d %d %d %d %ld %ld %d\n",
+		  TermWin_TotalWidth(), TermWin_TotalHeight(),
+		  r->TermWin.fwidth, r->TermWin.fheight,
+		  (long)GRX_SCALE * r->TermWin.fwidth / r->TermWin.width,
+		  (long)GRX_SCALE * r->TermWin.fheight / r->TermWin.height,
+		  XDEPTH);
 }
 
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_DestroyWindow(grwin_t *grwin)
+rxvt_Gr_DestroyWindow(rxvt_t *r, grwin_t *grwin)
 {
     grcmd_t        *cmd, *next;
 
@@ -214,22 +217,22 @@ Gr_DestroyWindow(grwin_t *grwin)
 	free(cmd);
     }
 
-    XDestroyWindow(Xdisplay, grwin->win);
+    XDestroyWindow(r->Xdisplay, grwin->win);
     if (grwin->next != NULL)
 	grwin->next->prev = grwin->prev;
     if (grwin->prev != NULL)
 	grwin->prev->next = grwin->next;
     else
-	gr_root = grwin->next;
+	r->h->gr_root = grwin->next;
     free(grwin);
 
-    graphics_up--;
+    r->h->graphics_up--;
 }
 
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_Dispatch(grwin_t *grwin, grcmd_t *data)
+rxvt_Gr_Dispatch(rxvt_t *r, grwin_t *grwin, grcmd_t *data)
 {
     int             i, n;
     union {
@@ -240,8 +243,8 @@ Gr_Dispatch(grwin_t *grwin, grcmd_t *data)
     if (data->color != Color_fg) {
 	XGCValues       gcv;
 
-	gcv.foreground = PixColors[data->color];
-	XChangeGC(Xdisplay, TermWin.gc, GCForeground, &gcv);
+	gcv.foreground = r->PixColors[data->color];
+	XChangeGC(r->Xdisplay, r->TermWin.gc, GCForeground, &gcv);
     }
     if (grwin)
 	switch (data->cmd) {
@@ -251,8 +254,8 @@ Gr_Dispatch(grwin_t *grwin, grcmd_t *data)
 		    xdata.pt[n].x = data->coords[i] * grwin->w / GRX_SCALE;
 		    xdata.pt[n].y = data->coords[i + 1] * grwin->h / GRX_SCALE;
 		}
-		XDrawLines(Xdisplay,
-		       grwin->win, TermWin.gc, xdata.pt, n, CoordModeOrigin);
+		XDrawLines(r->Xdisplay,
+		       grwin->win, r->TermWin.gc, xdata.pt, n, CoordModeOrigin);
 	    }
 	    break;
 
@@ -262,8 +265,8 @@ Gr_Dispatch(grwin_t *grwin, grcmd_t *data)
 		    xdata.pt[n].x = data->coords[i] * grwin->w / GRX_SCALE;
 		    xdata.pt[n].y = data->coords[i + 1] * grwin->h / GRX_SCALE;
 		}
-		XDrawPoints(Xdisplay,
-		       grwin->win, TermWin.gc, xdata.pt, n, CoordModeOrigin);
+		XDrawPoints(r->Xdisplay,
+		       grwin->win, r->TermWin.gc, xdata.pt, n, CoordModeOrigin);
 	    }
 	    break;
 
@@ -279,39 +282,39 @@ Gr_Dispatch(grwin_t *grwin, grcmd_t *data)
 		    xdata.rect[n].height = ((data->coords[i + 3]
 					     - data->coords[i + 1] + 1) *
 					    grwin->h / GRX_SCALE);
-		    XPMClearArea(Xdisplay, grwin->win,
+		    XPMClearArea(r->Xdisplay, grwin->win,
 				 xdata.rect[n].x, xdata.rect[n].y,
 				 xdata.rect[n].width, xdata.rect[n].height,
 				 0);
 		}
-		XFillRectangles(Xdisplay, grwin->win, TermWin.gc, xdata.rect,
+		XFillRectangles(r->Xdisplay, grwin->win, r->TermWin.gc, xdata.rect,
 				n);
 	    }
 	    break;
 	case 'T':
-	    Gr_Text(grwin, data);
+	    rxvt_Gr_Text(r, grwin, data);
 	    break;
 	case 'C':
-	    Gr_ClearWindow(grwin);
+	    rxvt_Gr_ClearWindow(r, grwin);
 	    break;
 	}
     if (data->color != Color_fg) {
 	XGCValues       gcv;
 
-	gcv.foreground = PixColors[Color_fg];
-	XChangeGC(Xdisplay, TermWin.gc, GCForeground, &gcv);
+	gcv.foreground = r->PixColors[Color_fg];
+	XChangeGC(r->Xdisplay, r->TermWin.gc, GCForeground, &gcv);
     }
 }
 
 /* ARGSUSED */
 /* INTPROTO */
 void
-Gr_Redraw(grwin_t *grwin)
+rxvt_Gr_Redraw(rxvt_t *r, grwin_t *grwin)
 {
     grcmd_t        *cmd;
 
     for (cmd = grwin->graphics; cmd != NULL; cmd = cmd->next)
-	Gr_Dispatch(grwin, cmd);
+	rxvt_Gr_Dispatch(r, grwin, cmd);
 }
 
 /*----------------------------------------------------------------------*
@@ -320,15 +323,15 @@ Gr_Redraw(grwin_t *grwin)
 /* ARGSUSED */
 /* EXTPROTO */
 void
-Gr_ButtonReport(int but, int x, int y)
+rxvt_Gr_ButtonReport(rxvt_t *r, int but, int x, int y)
 {
     grwin_t        *grwin;
 
-    for (grwin = gr_root; grwin != NULL; grwin = grwin->next)
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = grwin->next)
 	if ((x > grwin->x)
 	    && (y > grwin->y)
-	    && (x < grwin->x + grwin->w)
-	    && (y < grwin->y + grwin->h))
+	    && ((unsigned int)x < grwin->x + grwin->w)
+	    && ((unsigned int)y < grwin->y + grwin->h))
 	    break;
 
     if (grwin == NULL)
@@ -336,37 +339,36 @@ Gr_ButtonReport(int but, int x, int y)
 
     x = GRX_SCALE * (x - grwin->x) / grwin->w;
     y = GRX_SCALE * (y - grwin->y) / grwin->h;
-    tt_printf("\033%c%ld;%d;%d;\n", but, (long)grwin->win, x, y);
+    rxvt_tt_printf(r, "\033%c%ld;%d;%d;\n", but, (long)grwin->win, x, y);
 }
 
 /* ARGSUSED */
 /* EXTPROTO */
 void
-Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
+rxvt_Gr_do_graphics(rxvt_t *r, int cmd, unsigned int nargs, int args[], unsigned char *text)
 {
-    static Window   last_id = None;
-    long            win_id;
+    int             i;
+    Window          win_id;
     grwin_t        *grwin;
     grcmd_t        *newcmd, *oldcmd;
-    int             i;
 
     if (cmd == 'W') {
-	Gr_NewWindow(nargs, args);
+	rxvt_Gr_NewWindow(r, nargs, args);
 	return;
     }
     win_id = (nargs > 0) ? (Window) args[0] : None;
 
     if ((cmd == 'G') && (win_id == None)) {
-	Gr_Geometry(NULL, NULL);
+	rxvt_Gr_Geometry(r, NULL, NULL);
 	return;
     }
-    if ((win_id == None) && (last_id != None))
-	win_id = last_id;
+    if ((win_id == None) && (r->h->gr_last_id != None))
+	win_id = r->h->gr_last_id;
 
     if (win_id == None)
 	return;
 
-    grwin = gr_root;
+    grwin = r->h->gr_root;
     while ((grwin != NULL) && (grwin->win != win_id))
 	grwin = grwin->next;
 
@@ -374,20 +376,20 @@ Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
 	return;
 
     if (cmd == 'G') {
-	Gr_Geometry(grwin, NULL);
+	rxvt_Gr_Geometry(r, grwin, NULL);
 	return;
     }
     nargs--;
     args++;			/* skip over window id */
 
 /* record this new command */
-    newcmd = (grcmd_t *) MALLOC(sizeof(grcmd_t));
+    newcmd = (grcmd_t *) malloc(sizeof(grcmd_t));
     newcmd->ncoords = nargs;
-    newcmd->coords = (int *)MALLOC((newcmd->ncoords * sizeof(int)));
+    newcmd->coords = (int *)malloc((newcmd->ncoords * sizeof(int)));
 
     newcmd->next = NULL;
     newcmd->cmd = cmd;
-    newcmd->color = scr_get_fgcolor();
+    newcmd->color = rxvt_scr_get_fgcolor(r);
     newcmd->text = text;
 
     for (i = 0; i < newcmd->ncoords; i++)
@@ -424,47 +426,47 @@ Gr_do_graphics(int cmd, int nargs, int args[], unsigned char *text)
 	    oldcmd = oldcmd->next;
 	oldcmd->next = newcmd;
     }
-    Gr_Dispatch(grwin, newcmd);
+    rxvt_Gr_Dispatch(r, grwin, newcmd);
 }
 
 /* ARGSUSED */
 /* EXTPROTO */
 void
-Gr_scroll(int count)
+rxvt_Gr_scroll(rxvt_t *r, int count)
 {
-    static short    prev_start = 0;
     grwin_t        *grwin, *next;
 
-    if ((count == 0) && (prev_start == TermWin.view_start))
+    if ((count == 0) && (r->h->gr_prev_start == r->TermWin.view_start))
 	return;
 
-    prev_start = TermWin.view_start;
+    r->h->gr_prev_start = r->TermWin.view_start;
 
-    for (grwin = gr_root; grwin != NULL; grwin = next) {
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = next) {
 	next = grwin->next;
-	grwin->y -= (count * TermWin.fheight);
-	if ((grwin->y + grwin->h) < -(TermWin.saveLines * TermWin.fheight))
-	    Gr_DestroyWindow(grwin);
+	grwin->y -= (count * r->TermWin.fheight);
+	if ((long)(grwin->y + grwin->h)
+	    < -(long)(r->TermWin.saveLines * r->TermWin.fheight))
+	    rxvt_Gr_DestroyWindow(r, grwin);
 	else
-	    XMoveWindow(Xdisplay, grwin->win,
+	    XMoveWindow(r->Xdisplay, grwin->win,
 			grwin->x,
-			grwin->y + (TermWin.view_start * TermWin.fheight));
+			grwin->y + (r->TermWin.view_start * r->TermWin.fheight));
     }
 }
 
 /* EXTPROTO */
 void
-Gr_ClearScreen(void)
+rxvt_Gr_ClearScreen(rxvt_t *r)
 {
     grwin_t        *grwin, *next;
 
-    for (grwin = gr_root; grwin != NULL; grwin = next) {
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = next) {
 	next = grwin->next;
 	if ((grwin->screen == 0) && (grwin->y + grwin->h > 0)) {
 	    if (grwin->y >= 0)
-		Gr_DestroyWindow(grwin);
+		rxvt_Gr_DestroyWindow(r, grwin);
 	    else
-		XResizeWindow(Xdisplay, grwin->win,
+		XResizeWindow(r->Xdisplay, grwin->win,
 			      grwin->w, -grwin->y);
 	}
     }
@@ -472,18 +474,18 @@ Gr_ClearScreen(void)
 
 /* EXTPROTO */
 void
-Gr_ChangeScreen(void)
+rxvt_Gr_ChangeScreen(rxvt_t *r)
 {
     grwin_t        *grwin, *next;
 
-    for (grwin = gr_root; grwin != NULL; grwin = next) {
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = next) {
 	next = grwin->next;
 	if (grwin->y + grwin->h > 0) {
 	    if (grwin->screen == 1) {
-		XMapWindow(Xdisplay, grwin->win);
+		XMapWindow(r->Xdisplay, grwin->win);
 		grwin->screen = 0;
 	    } else {
-		XUnmapWindow(Xdisplay, grwin->win);
+		XUnmapWindow(r->Xdisplay, grwin->win);
 		grwin->screen = 1;
 	    }
 	}
@@ -493,13 +495,13 @@ Gr_ChangeScreen(void)
 /* ARGSUSED */
 /* EXTPROTO */
 void
-Gr_expose(Window win)
+rxvt_Gr_expose(rxvt_t *r, Window win)
 {
     grwin_t        *grwin;
 
-    for (grwin = gr_root; grwin != NULL; grwin = grwin->next) {
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = grwin->next) {
 	if (grwin->win == win) {
-	    Gr_Redraw(grwin);
+	    rxvt_Gr_Redraw(r, grwin);
 	    break;
 	}
     }
@@ -508,40 +510,40 @@ Gr_expose(Window win)
 /* ARGSUSED */
 /* EXTPROTO */
 void
-Gr_Resize(int w, int h)
+rxvt_Gr_Resize(rxvt_t *r, int w __attribute__((unused)), int h)
 {
     grwin_t        *grwin;
 
-    for (grwin = gr_root; grwin != NULL; grwin = grwin->next) {
-	if (TermWin.height != h) {
-	    grwin->y += (TermWin.height - h);
-	    XMoveWindow(Xdisplay, grwin->win,
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = grwin->next) {
+	if (r->TermWin.height != h) {
+	    grwin->y += (r->TermWin.height - h);
+	    XMoveWindow(r->Xdisplay, grwin->win,
 			grwin->x,
-			grwin->y + (TermWin.view_start * TermWin.fheight));
+			grwin->y + (r->TermWin.view_start * r->TermWin.fheight));
 	}
-	Gr_Redraw(grwin);
+	rxvt_Gr_Redraw(r, grwin);
     }
 }
 
 /* EXTPROTO */
 void
-Gr_reset(void)
+rxvt_Gr_reset(rxvt_t *r)
 {
     grwin_t        *grwin, *next;
 
-    for (grwin = gr_root; grwin != NULL; grwin = next) {
+    for (grwin = r->h->gr_root; grwin != NULL; grwin = next) {
 	next = grwin->next;
-	Gr_DestroyWindow(grwin);
+	rxvt_Gr_DestroyWindow(r, grwin);
     }
 
-    graphics_up = 0;
+    r->h->graphics_up = 0;
 }
 
 /* EXTPROTO */
 int
-Gr_Displayed(void)
+rxvt_Gr_Displayed(rxvt_t *r)
 {
-    return graphics_up;
+    return r->h->graphics_up;
 }
 #endif
 /*----------------------- end-of-file (C source) -----------------------*/
