@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	logging.c
  *----------------------------------------------------------------------*
- * $Id: logging.c,v 1.18 1999/12/18 12:19:10 mason Exp $
+ * $Id: logging.c,v 1.25 2001/04/02 17:04:55 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1992      John Bovey <jdb@ukc.ac.uk>
@@ -11,7 +11,8 @@
  * Copyright (C) 1995      Raul Garcia Garcia <rgg@tid.es>
  * Copyright (C) 1995      Piet W. Plomp <piet@idefix.icce.rug.nl>
  * Copyright (C) 1997      Raul Garcia Garcia <rgg@tid.es>
- * Copyright (C) 1997,1999 Geoff Wing <gcw@pobox.com>
+ * Copyright (C) 1998-2001 Geoff Wing <gcw@pobox.com>
+ * 				- extensive modifications
  * Copyright (C) 1999      D J Hawkey Jr <hawkeyd@visi.com>
  *				- lastlog support
  *
@@ -57,12 +58,6 @@
  * SYSV style utmp entry
  *      ut_user, ut_id, ut_line, ut_pid, ut_type, ut_exit, ut_time
  */
-static UTMP     ut;
-static int      utmp_pos = 0;
-
-#ifdef USE_SYSV_UTMP
-static char     ut_id[5];
-#endif
 
 /*
  * update wtmp entries
@@ -110,17 +105,17 @@ rxvt_update_wtmp(const char *fname, const UTMP *putmp)
 #ifndef USE_SYSV_UTMP
 /* INTPROTO */
 int
-write_bsd_utmp(UTMP wu)
+rxvt_write_bsd_utmp(int utmp_pos, UTMP *wu)
 {
     int             fd;
 
-    if (utmp_pos > 0 && (fd = open(RXVT_REAL_UTMP_FILE, O_WRONLY)) != -1) {
-	lseek(fd, (off_t) (utmp_pos * sizeof(UTMP)), SEEK_SET);
-	write(fd, &wu, sizeof(UTMP));
-	close(fd);
-	return 1;
-    }
-    return 0;
+    if (utmp_pos <= 0 || (fd = open(RXVT_REAL_UTMP_FILE, O_WRONLY)) == -1)
+	return 0;
+
+    lseek(fd, (off_t) (utmp_pos * sizeof(UTMP)), SEEK_SET);
+    write(fd, wu, sizeof(UTMP));
+    close(fd);
+    return 1;
 }
 #endif
 /* ------------------------------------------------------------------------- */
@@ -140,7 +135,7 @@ rxvt_update_lastlog(const char *fname, const char *pty, const char *host)
 
     pwent = getpwuid(getuid());
     if (!pwent) {
-	print_error("no entry in password file");
+	rxvt_print_error("no entry in password file");
 	return;
     }
     MEMSET(&ll, 0, sizeof(ll));
@@ -174,38 +169,39 @@ rxvt_update_lastlog(const char *fname, const char *pty, const char *host)
  */
 /* EXTPROTO */
 void
-makeutent(const char *pty, const char *hostname)
+rxvt_makeutent(rxvt_t *r, const char *pty, const char *hostname)
 #ifndef USE_SYSV_UTMP
 /* ------------------------------ BSD ------------------------------  */
 {
     FILE           *fd0;
+    UTMP           *ut = &(r->h->ut);
     struct passwd  *pwent = getpwuid(getuid());
-    char            buf[256], name[256];
 #ifdef __QNX__
     UTMP            u2;
     struct utsname  un;
 #endif
+    char            buf[256], name[256];
 
 /* BSD naming is of the form /dev/tty?? or /dev/pty?? */
-    MEMSET(&ut, 0, sizeof(UTMP));
+    MEMSET(ut, 0, sizeof(UTMP));
 
     if (!STRNCMP(pty, "/dev/", 5))
 	pty += 5;		/* skip /dev/ prefix */
     if (STRNCMP(pty, "pty", 3) && STRNCMP(pty, "tty", 3)) {
-	print_error("can't parse tty name \"%s\"", pty);
+	rxvt_print_error("can't parse tty name \"%s\"", pty);
 	return;
     }
 #ifdef __QNX__
     uname(&un);
-    STRNCPY(ut.id, un.nodename, sizeof(ut.ut_line));
-    ut.ut_pid = getpid();
-    ut.ut_type = USER_PROCESS;
+    STRNCPY(ut->id, un.nodename, sizeof(ut->ut_line));
+    ut->ut_pid = getpid();
+    ut->ut_type = USER_PROCESS;
 #endif
-    STRNCPY(ut.ut_line, pty, sizeof(ut.ut_line));
-    STRNCPY(ut.ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut.ut_name));
-    STRNCPY(ut.ut_host, hostname, sizeof(ut.ut_host));
-    ut.ut_time = time(NULL);
+    STRNCPY(ut->ut_line, pty, sizeof(ut->ut_line));
+    STRNCPY(ut->ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(ut->ut_name));
+    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
+    ut->ut_time = time(NULL);
 
     buf[sizeof(buf) - 1] = '\0';
 #ifdef __QNX__
@@ -224,8 +220,8 @@ makeutent(const char *pty, const char *hostname)
 	for (i = 1; (fgets(buf, sizeof(buf) - 1, fd0) != NULL);) {
 	    if (*buf == '#' || sscanf(buf, "%s", name) != 1)
 		continue;
-	    if (!STRCMP(ut.ut_line, name)) {
-		utmp_pos = i;
+	    if (!STRCMP(ut->ut_line, name)) {
+		r->h->utmp_pos = i;
 		fclose(fd0);
 		break;
 	    }
@@ -234,16 +230,16 @@ makeutent(const char *pty, const char *hostname)
 	fclose(fd0);
     }
 #endif
-    if (!write_bsd_utmp(ut))
-	utmp_pos = 0;
+    if (!rxvt_write_bsd_utmp(r->h->utmp_pos, ut))
+	r->h->utmp_pos = 0;
 #ifdef WTMP_SUPPORT
 # ifdef WTMP_ONLY_ON_LOGIN
-    if (Options & Opt_loginShell)
+    if (r->Options & Opt_loginShell)
 # endif
-	update_wtmp(RXVT_WTMP_FILE, &ut);
+	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
 #endif
-#ifdef LASTLOG_SUPPORT
-    if (Options & Opt_loginShell)
+#if defined(LASTLOG_SUPPORT) && defined(RXVT_LASTLOG_FILE)
+    if (r->Options & Opt_loginShell)
 	rxvt_update_lastlog(RXVT_LASTLOG_FILE, pty, hostname);
 #endif
 }
@@ -252,18 +248,19 @@ makeutent(const char *pty, const char *hostname)
 {
     int             i;
     char           *colon;
+    UTMP           *ut = &(r->h->ut);
     struct passwd  *pwent = getpwuid(getuid());
 
-    MEMSET(&ut, 0, sizeof(UTMP));
+    MEMSET(ut, 0, sizeof(UTMP));
 
     if (!STRNCMP(pty, "/dev/", 5))
 	pty += 5;		/* skip /dev/ prefix */
     if (!STRNCMP(pty, "pty", 3) || !STRNCMP(pty, "tty", 3))
-	STRNCPY(ut.ut_id, (pty + 3), sizeof(ut.ut_id));
+	STRNCPY(ut->ut_id, (pty + 3), sizeof(ut->ut_id));
     else if (sscanf(pty, "pts/%d", &i) == 1)
-	sprintf(ut.ut_id, "vt%02x", (i & 0xff));	/* sysv naming */
+	sprintf(ut->ut_id, "vt%02x", (i & 0xff));	/* sysv naming */
     else {
-	print_error("can't parse tty name \"%s\"", pty);
+	rxvt_print_error("can't parse tty name \"%s\"", pty);
 	return;
     }
 
@@ -271,54 +268,54 @@ makeutent(const char *pty, const char *hostname)
     /* XXX: most likely unnecessary.  could be harmful */
     utmpname(RXVT_REAL_UTMP_FILE);
 #endif
-    STRNCPY(ut_id, ut.ut_id, sizeof(ut_id));
+    STRNCPY(r->h->ut_id, ut->ut_id, sizeof(r->h->ut_id));
 
     setutent();			/* XXX: should be unnecessaray */
 
-    ut.ut_type = DEAD_PROCESS;
-    getutid(&ut);		/* position to entry in utmp file */
+    ut->ut_type = DEAD_PROCESS;
+    getutid(ut);		/* position to entry in utmp file */
 
 /* set up the new entry */
-    ut.ut_type = USER_PROCESS;
+    ut->ut_type = USER_PROCESS;
 #ifndef linux
-    ut.ut_exit.e_exit = 2;
+    ut->ut_exit.e_exit = 2;
 #endif
-    STRNCPY(ut.ut_user, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut.ut_user));
+    STRNCPY(ut->ut_user, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(ut->ut_user));
 /* ut_name is normally the same as ut_user, but .... */
-    STRNCPY(ut.ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut.ut_name));
-    STRNCPY(ut.ut_id, ut_id, sizeof(ut.ut_id));
-    STRNCPY(ut.ut_line, pty, sizeof(ut.ut_line));
+    STRNCPY(ut->ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(ut->ut_name));
+    STRNCPY(ut->ut_id, r->h->ut_id, sizeof(ut->ut_id));
+    STRNCPY(ut->ut_line, pty, sizeof(ut->ut_line));
 
 #if (defined(HAVE_UTMP_HOST) && ! defined(RXVT_UTMP_AS_UTMPX)) || (defined(HAVE_UTMPX_HOST) && defined(RXVT_UTMP_AS_UTMPX))
-    STRNCPY(ut.ut_host, hostname, sizeof(ut.ut_host));
+    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
 # ifndef linux
-    if ((colon = STRRCHR(ut.ut_host, ':')) != NULL)
+    if ((colon = STRRCHR(ut->ut_host, ':')) != NULL)
 	*colon = '\0';
 # endif
 #endif
 
-    ut.ut_pid = getpid();
+    ut->ut_pid = getpid();
 
 #ifdef RXVT_UTMP_AS_UTMPX
-    ut.ut_session = getsid(0);
-    ut.ut_tv.tv_sec = time(NULL);
-    ut.ut_tv.tv_usec = 0;
+    ut->ut_session = getsid(0);
+    ut->ut_tv.tv_sec = time(NULL);
+    ut->ut_tv.tv_usec = 0;
 #else
-    ut.ut_time = time(NULL);
+    ut->ut_time = time(NULL);
 #endif				/* HAVE_UTMPX_H */
-    pututline(&ut);
-    utmp_pos = 1;
+    pututline(ut);
+    r->h->utmp_pos = 1;
 
 #ifdef WTMP_SUPPORT
 # ifdef WTMP_ONLY_ON_LOGIN
-    if (Options & Opt_loginShell)
+    if (r->Options & Opt_loginShell)
 # endif
-	update_wtmp(RXVT_WTMP_FILE, &ut);
+	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
 #endif
-#ifdef LASTLOG_SUPPORT
-    if (Options & Opt_loginShell)
+#if defined(LASTLOG_SUPPORT) && defined(RXVT_LASTLOG_FILE)
+    if (r->Options & Opt_loginShell)
 	rxvt_update_lastlog(RXVT_LASTLOG_FILE, pty, hostname);
 #endif
     endutent();			/* close the file */
@@ -331,10 +328,11 @@ makeutent(const char *pty, const char *hostname)
  */
 /* EXTPROTO */
 void
-cleanutent(void)
+rxvt_cleanutent(rxvt_t *r)
 #ifndef USE_SYSV_UTMP
 /* ------------------------------ BSD ------------------------------  */
 {
+    UTMP           *ut = &(r->h->ut);
 #ifdef __QNX__
     UTMP            u2;
 
@@ -342,44 +340,44 @@ cleanutent(void)
 #endif
 #ifdef WTMP_SUPPORT
 # ifdef WTMP_ONLY_ON_LOGIN
-    if (Options & Opt_loginShell)
+    if (r->Options & Opt_loginShell)
 # endif
     {
-	MEMSET(&ut.ut_name, 0, sizeof(ut.ut_name));
-	MEMSET(&ut.ut_host, 0, sizeof(ut.ut_host));
-	ut.ut_time = time(NULL);
-	update_wtmp(RXVT_WTMP_FILE, &ut);
+	MEMSET(ut->ut_name, 0, sizeof(ut->ut_name));
+	MEMSET(ut->ut_host, 0, sizeof(ut->ut_host));
+	ut->ut_time = time(NULL);
+	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
     }
 #endif
-    if (!utmp_pos)
+    if (!r->h->utmp_pos)
 	return;
 #ifdef __QNX__
     u2.ut_type = DEAD_PROCESS;
-    write_bsd_utmp(u2);
+    rxvt_write_bsd_utmp(r->h->utmp_pos, &u2);
 #else
-    MEMSET(&ut, 0, sizeof(UTMP));
-    write_bsd_utmp(ut);
+    MEMSET(ut, 0, sizeof(UTMP));
+    rxvt_write_bsd_utmp(r->h->utmp_pos, ut);
 #endif
 }
 #else				/* USE_SYSV_UTMP */
 /* ------------------------------ SYSV ------------------------------  */
 {
-    UTMP           *putmp;
+    UTMP           *putmp, *ut = &(r->h->ut);
 
-    if (!utmp_pos)
+    if (!r->h->utmp_pos)
 	return;
 
 #if 0
     /* XXX: most likely unnecessary.  could be harmful */
     utmpname(RXVT_REAL_UTMP_FILE);
 #endif
-    MEMSET(&ut, 0, sizeof(UTMP));
-    STRNCPY(ut.ut_id, ut_id, sizeof(ut.ut_id));
-    ut.ut_type = USER_PROCESS;
+    MEMSET(ut, 0, sizeof(UTMP));
+    STRNCPY(ut->ut_id, r->h->ut_id, sizeof(ut->ut_id));
+    ut->ut_type = USER_PROCESS;
 
     setutent();			/* XXX: should be unnecessaray */
 
-    putmp = getutid(&ut);
+    putmp = getutid(ut);
     if (!putmp || putmp->ut_pid != getpid())
 	return;
 
@@ -396,9 +394,9 @@ cleanutent(void)
 
 #ifdef WTMP_SUPPORT
 # ifdef WTMP_ONLY_ON_LOGIN
-    if (Options & Opt_loginShell)
+    if (r->Options & Opt_loginShell)
 # endif
-	update_wtmp(RXVT_WTMP_FILE, putmp);
+	update_wtmp(RXVT_REAL_WTMP_FILE, putmp);
 #endif
 
     endutent();
