@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	netdisp.c
  *----------------------------------------------------------------------*
- * $Id: netdisp.c,v 1.12 1999/07/04 15:10:17 mason Exp $
+ * $Id: netdisp.c,v 1.14 2001/04/05 10:41:36 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1996      Chuck Blake <cblake@BBN.COM>
@@ -42,7 +42,7 @@
 /* return NULL a pointer to buffer which may be freed */
 /* EXTPROTO */
 char           *
-network_display(const char *display)
+rxvt_network_display(const char *display)
 {
     char            buffer[1024], *rval = NULL;
     struct ifconf   ifc;
@@ -86,7 +86,7 @@ network_display(const char *display)
 		if (colon == NULL)
 		    colon = ":0.0";
 
-		rval = MALLOC(STRLEN(colon) + 16);
+		rval = malloc(STRLEN(colon) + 16);
 		sprintf(rval, "%d.%d.%d.%d%s",
 			(int)((addr >> 030) & 0xFF),
 			(int)((addr >> 020) & 0xFF),
