@@ -1,5 +1,5 @@
 /*
- * $Id: screen.h,v 1.15 2000/02/21 00:36:47 mason Exp $
+ * $Id: screen.h,v 1.17 2001/04/05 14:26:50 gcw Exp $
  */
 
 #ifndef _SCREEN_H_		/* include once only */
@@ -57,19 +57,19 @@ typedef struct {
     int16_t	   *tlen;	/* length of each text line                  */
     rend_t        **rend;	/* rendition, uses RS_ flags                 */
     row_col_t       cur;	/* cursor position on the screen             */
-    int16_t         tscroll,	/* top of settable scroll region             */
+    u_int16_t       tscroll,	/* top of settable scroll region             */
                     bscroll,	/* bottom of settable scroll region          */
 		    charset;	/* character set number [0..3]               */
     unsigned int    flags;	/* see below                                 */
     row_col_t	    s_cur;	/* saved cursor position                     */
-    int16_t	    s_charset;	/* saved character set number [0..3]         */
+    u_int16_t	    s_charset;	/* saved character set number [0..3]         */
     char            s_charset_char;
     rend_t          s_rstyle;	/* saved rendition style                     */
 } screen_t;
 
 typedef struct {
     unsigned char  *text;	/* selected text                             */
-    int             len;	/* length of selected text                   */
+    u_int32_t       len;	/* length of selected text                   */
     enum {
 	SELECTION_CLEAR = 0,	/* nothing selected                          */
 	SELECTION_INIT,		/* marked a point                            */
@@ -112,30 +112,4 @@ typedef struct {
     char          *name;
 } KNOWN_ENCODINGS;
 
-/* ------------------------------------------------------------------------- *
- *                             MODULE VARIABLES                              *
- * ------------------------------------------------------------------------- */
-
-#ifdef INTERN_SCREEN
-# define EXTSCR
-#else
-# define EXTSCR	extern
-#endif
-
-/* This tells what's actually on the screen */
-EXTSCR text_t    **drawn_text;
-EXTSCR rend_t    **drawn_rend;
-EXTSCR text_t    **buf_text;
-EXTSCR rend_t    **buf_rend;
-EXTSCR int16_t    *buf_tlen;
-EXTSCR char       *tabs;	/* a 1 for a location with a tab-stop */
-EXTSCR screen_t    screen;
-EXTSCR screen_t    swap;
-EXTSCR selection_t selection;
-EXTSCR sstyle_t    selection_style;
-#ifdef MULTICHAR_SET
-EXTSCR ENC_METHOD  encoding_method;
-#endif
-
-
 #endif /* _SCREEN_H_ */
