@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	xdefaults.c
  *----------------------------------------------------------------------*
- * $Id: xdefaults.c,v 1.57 2000/03/11 05:14:50 mason Exp $
+ * $Id: xdefaults.c,v 1.86 2001/04/09 03:34:08 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1994      Robert Nation <nation@rocket.sanders.lockheed.com>
@@ -40,32 +40,32 @@ static const char *const xnames[2] = { ".Xdefaults", ".Xresources" };
 /*
  * `string' options MUST have a usage argument
  * `switch' and `boolean' options have no argument
- * if there's no desc(ription), it won't appear in usage()
+ * if there's no desc(ription), it won't appear in rxvt_usage()
  */
 
 /* INFO() - descriptive information only */
 #define INFO(opt, arg, desc)					\
-    {0, NULL, NULL, (opt), (arg), (desc)}
+    {0, -1, NULL, (opt), (arg), (desc)}
 
 /* STRG() - command-line option, with/without resource */
 #define STRG(rsp, kw, opt, arg, desc)				\
-    {0, &(rsp), (kw), (opt), (arg), (desc)}
+    {0, (rsp), (kw), (opt), (arg), (desc)}
 
 /* RSTRG() - resource/long-option */
 #define RSTRG(rsp, kw, arg)					\
-    {0, &(rsp), (kw), NULL, (arg), NULL}
+    {0, (rsp), (kw), NULL, (arg), NULL}
 
 /* BOOL() - regular boolean `-/+' flag */
 #define BOOL(rsp, kw, opt, flag, desc)				\
-    {(Opt_Boolean|(flag)), &(rsp), (kw), (opt), NULL, (desc)}
+    {(Opt_Boolean|(flag)), (rsp), (kw), (opt), NULL, (desc)}
 
 /* SWCH() - `-' flag */
 #define SWCH(opt, flag, desc)					\
-    {(flag), NULL, NULL, (opt), NULL, (desc)}
+    {(flag), -1, NULL, (opt), NULL, (desc)}
 
 /* convenient macros */
-#define optList_STRLEN(i)						\
-    (optList[i].flag ? 0 : (optList[i].arg ? STRLEN (optList[i].arg) : 1))
+#define optList_strlen(i)						\
+    (optList[i].flag ? 0 : (optList[i].arg ? STRLEN(optList[i].arg) : 1))
 #define optList_isBool(i)						\
     (optList[i].flag & Opt_Boolean)
 #define optList_isReverse(i)						\
@@ -75,196 +75,210 @@ static const char *const xnames[2] = { ".Xdefaults", ".Xresources" };
 
 static const struct {
     const unsigned long flag;	/* Option flag */
-    const char    **dp;		/* data pointer */
+    const int       doff;	/* data offset */
     const char     *kw;		/* keyword */
     const char     *opt;	/* option */
     const char     *arg;	/* argument */
     const char     *desc;	/* description */
 } optList[] = {
-    STRG(rs.display_name, NULL, "d", NULL, NULL),	/* short form */
-    STRG(rs.display_name, NULL, "display", "string", "X server to contact"),
-    STRG(rs.term_name, "termName", "tn", "string",
+    STRG(Rs_display_name, NULL, "d", NULL, NULL),	/* short form */
+    STRG(Rs_display_name, NULL, "display", "string", "X server to contact"),
+    STRG(Rs_term_name, "termName", "tn", "string",
 	 "value of the TERM environment variable"),
-    STRG(rs.geometry, NULL, "g", NULL, NULL),	/* short form */
-    STRG(rs.geometry, "geometry", "geometry", "geometry",
+    STRG(Rs_geometry, NULL, "g", NULL, NULL),	/* short form */
+    STRG(Rs_geometry, "geometry", "geometry", "geometry",
 	 "size (in characters) and position"),
     SWCH("C", Opt_console, "intercept console messages"),
     SWCH("iconic", Opt_iconic, "start iconic"),
     SWCH("ic", Opt_iconic, NULL),	/* short form */
-    BOOL(rs.reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
+    BOOL(Rs_reverseVideo, "reverseVideo", "rv", Opt_reverseVideo,
 	 "reverse video"),
-    BOOL(rs.loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
-    BOOL(rs.scrollBar, "scrollBar", "sb", Opt_scrollBar, "scrollbar"),
-    BOOL(rs.scrollBar_right, "scrollBar_right", "sr", Opt_scrollBar_right,
+    BOOL(Rs_loginShell, "loginShell", "ls", Opt_loginShell, "login shell"),
+#ifdef HAVE_SCROLLBARS
+    BOOL(Rs_scrollBar, "scrollBar", "sb", Opt_scrollBar, "scrollbar"),
+    BOOL(Rs_scrollBar_right, "scrollBar_right", "sr", Opt_scrollBar_right,
 	 "scrollbar right"),
-    BOOL(rs.scrollBar_floating, "scrollBar_floating", "st",
+    BOOL(Rs_scrollBar_floating, "scrollBar_floating", "st",
 	 Opt_scrollBar_floating, "scrollbar without a trough"),
-    BOOL(rs.scrollTtyOutput, "scrollTtyOutput", NULL, Opt_scrollTtyOutput,
+#endif
+    BOOL(Rs_scrollTtyOutput, "scrollTtyOutput", NULL, Opt_scrollTtyOutput,
 	 NULL),
-    BOOL(rs.scrollTtyOutput, NULL, "si", Opt_Reverse | Opt_scrollTtyOutput,
+    BOOL(Rs_scrollTtyOutput, NULL, "si", Opt_Reverse | Opt_scrollTtyOutput,
 	 "scroll-on-tty-output inhibit"),
-    BOOL(rs.scrollKeypress, "scrollTtyKeypress", "sk", Opt_scrollKeypress,
+    BOOL(Rs_scrollTtyKeypress, "scrollTtyKeypress", "sk", Opt_scrollTtyKeypress,
 	 "scroll-on-keypress"),
 #ifdef TRANSPARENT
-    BOOL(rs.transparent, "inheritPixmap", "ip", Opt_transparent,
+    BOOL(Rs_transparent, "inheritPixmap", "ip", Opt_transparent,
 	 "inherit parent pixmap"),
     SWCH("tr", Opt_transparent, NULL),
 #endif
-    BOOL(rs.utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit, "utmp inhibit"),
+    BOOL(Rs_utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit, "utmp inhibit"),
 #ifndef NO_BELL
-    BOOL(rs.visualBell, "visualBell", "vb", Opt_visualBell, "visual bell"),
+    BOOL(Rs_visualBell, "visualBell", "vb", Opt_visualBell, "visual bell"),
 # if ! defined(NO_MAPALERT) && defined(MAPALERT_OPTION)
-    BOOL(rs.mapAlert, "mapAlert", NULL, Opt_mapAlert, NULL),
+    BOOL(Rs_mapAlert, "mapAlert", NULL, Opt_mapAlert, NULL),
 # endif
 #endif
 #ifdef META8_OPTION
-    BOOL(rs.meta8, "meta8", NULL, Opt_meta8, NULL),
+    BOOL(Rs_meta8, "meta8", NULL, Opt_meta8, NULL),
 #endif
-    STRG(rs.color[Color_bg], "background", "bg", "color", "background color"),
-    STRG(rs.color[Color_fg], "foreground", "fg", "color", "foreground color"),
-    RSTRG(rs.color[minCOLOR + 0], "color0", "color"),
-    RSTRG(rs.color[minCOLOR + 1], "color1", "color"),
-    RSTRG(rs.color[minCOLOR + 2], "color2", "color"),
-    RSTRG(rs.color[minCOLOR + 3], "color3", "color"),
-    RSTRG(rs.color[minCOLOR + 4], "color4", "color"),
-    RSTRG(rs.color[minCOLOR + 5], "color5", "color"),
-    RSTRG(rs.color[minCOLOR + 6], "color6", "color"),
-    RSTRG(rs.color[minCOLOR + 7], "color7", "color"),
+#ifdef MULTICHAR_SET
+    BOOL(Rs_mc_hack, "multibyte_cursor", "mcc", Opt_mc_hack,
+	 "Multibyte character cursor movement"),
+#endif
+#ifndef NO_FRILLS
+    BOOL(Rs_tripleclickwords, "tripleclickwords", "tcw", Opt_tripleclickwords,
+	 "triple click word selection"),
+#endif
+    STRG(Rs_color + Color_bg, "background", "bg", "color", "background color"),
+    STRG(Rs_color + Color_fg, "foreground", "fg", "color", "foreground color"),
+    RSTRG(Rs_color + minCOLOR + 0, "color0", "color"),
+    RSTRG(Rs_color + minCOLOR + 1, "color1", "color"),
+    RSTRG(Rs_color + minCOLOR + 2, "color2", "color"),
+    RSTRG(Rs_color + minCOLOR + 3, "color3", "color"),
+    RSTRG(Rs_color + minCOLOR + 4, "color4", "color"),
+    RSTRG(Rs_color + minCOLOR + 5, "color5", "color"),
+    RSTRG(Rs_color + minCOLOR + 6, "color6", "color"),
+    RSTRG(Rs_color + minCOLOR + 7, "color7", "color"),
 #ifndef NO_BRIGHTCOLOR
-    RSTRG(rs.color[minBrightCOLOR + 0], "color8", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 1], "color9", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 2], "color10", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 3], "color11", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 4], "color12", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 5], "color13", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 6], "color14", "color"),
-    RSTRG(rs.color[minBrightCOLOR + 7], "color15", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 0, "color8", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 1, "color9", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 2, "color10", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 3, "color11", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 4, "color12", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 5, "color13", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 6, "color14", "color"),
+    RSTRG(Rs_color + minBrightCOLOR + 7, "color15", "color"),
 #endif				/* NO_BRIGHTCOLOR */
-#ifndef NO_BOLDUNDERLINE
-    RSTRG(rs.color[Color_BD], "colorBD", "color"),
-    RSTRG(rs.color[Color_UL], "colorUL", "color"),
-#endif				/* NO_BOLDUNDERLINE */
+#ifndef NO_BOLD_UNDERLINE_REVERSE
+    RSTRG(Rs_color + Color_BD, "colorBD", "color"),
+    RSTRG(Rs_color + Color_UL, "colorUL", "color"),
+    RSTRG(Rs_color + Color_RV, "colorRV", "color"),
+#endif				/* ! NO_BOLD_UNDERLINE_REVERSE */
 #ifdef KEEP_SCROLLCOLOR
-    RSTRG(rs.color[Color_scroll], "scrollColor", "color"),
-    RSTRG(rs.color[Color_trough], "troughColor", "color"),
+    RSTRG(Rs_color + Color_scroll, "scrollColor", "color"),
+    RSTRG(Rs_color + Color_trough, "troughColor", "color"),
 #endif				/* KEEP_SCROLLCOLOR */
 #if defined (XPM_BACKGROUND) || (MENUBAR_MAX)
-    RSTRG(rs.path, "path", "search path"),
+    RSTRG(Rs_path, "path", "search path"),
 #endif				/* defined (XPM_BACKGROUND) || (MENUBAR_MAX) */
 #ifdef XPM_BACKGROUND
-    STRG(rs.backgroundPixmap, "backgroundPixmap", "pixmap", "file[;geom]",
+    STRG(Rs_backgroundPixmap, "backgroundPixmap", "pixmap", "file[;geom]",
 	 "background pixmap"),
 #endif				/* XPM_BACKGROUND */
 #if (MENUBAR_MAX)
-    RSTRG(rs.menu, "menu", "name[;tag]"),
+    RSTRG(Rs_menu, "menu", "name[;tag]"),
 #endif
 #ifndef NO_BOLDFONT
-    STRG(rs.boldFont, "boldFont", "fb", "fontname", "bold text font"),
+    STRG(Rs_boldFont, "boldFont", "fb", "fontname", "bold text font"),
 #endif
-    STRG(rs.font[0], "font", "fn", "fontname", "normal text font"),
+    STRG(Rs_font + 0, "font", "fn", "fontname", "normal text font"),
 /* fonts: command-line option = resource name */
-#if NFONTS > 1
-    RSTRG(rs.font[1], "font1", "fontname"),
+#if MAX_NFONTS > 1
+    RSTRG(Rs_font + 1, "font1", "fontname"),
 #endif
-#if NFONTS > 2
-    RSTRG(rs.font[2], "font2", "fontname"),
+#if MAX_NFONTS > 2
+    RSTRG(Rs_font + 2, "font2", "fontname"),
 #endif
-#if NFONTS > 3
-    RSTRG(rs.font[3], "font3", "fontname"),
+#if MAX_NFONTS > 3
+    RSTRG(Rs_font + 3, "font3", "fontname"),
 #endif
-#if NFONTS > 4
-    RSTRG(rs.font[4], "font4", "fontname"),
+#if MAX_NFONTS > 4
+    RSTRG(Rs_font + 4, "font4", "fontname"),
 #endif
-#if NFONTS > 5
-    RSTRG(rs.font[5], "font5", "fontname"),
+#if MAX_NFONTS > 5
+    RSTRG(Rs_font + 5, "font5", "fontname"),
 #endif
-#if NFONTS > 6
-    RSTRG(rs.font[6], "font6", "fontname"),
+#if MAX_NFONTS > 6
+    RSTRG(Rs_font + 6, "font6", "fontname"),
 #endif
-#if NFONTS > 7
-    RSTRG(rs.font[7], "font7", "fontname"),
+#if MAX_NFONTS > 7
+    RSTRG(Rs_font + 7, "font7", "fontname"),
 #endif
 #ifdef MULTICHAR_SET
-    STRG(rs.mfont[0], "mfont", "fm", "fontname", "multichar font"),
+    STRG(Rs_mfont + 0, "mfont", "fm", "fontname", "multichar font"),
 /* fonts: command-line option = resource name */
-# if NFONTS > 1
-    RSTRG(rs.mfont[1], "mfont1", "fontname"),
+# if MAX_NFONTS > 1
+    RSTRG(Rs_mfont + 1, "mfont1", "fontname"),
 # endif
-# if NFONTS > 2
-    RSTRG(rs.mfont[2], "mfont2", "fontname"),
+# if MAX_NFONTS > 2
+    RSTRG(Rs_mfont + 2, "mfont2", "fontname"),
 # endif
-# if NFONTS > 3
-    RSTRG(rs.mfont[3], "mfont3", "fontname"),
+# if MAX_NFONTS > 3
+    RSTRG(Rs_mfont + 3, "mfont3", "fontname"),
 # endif
-# if NFONTS > 4
-    RSTRG(rs.mfont[4], "mfont4", "fontname"),
+# if MAX_NFONTS > 4
+    RSTRG(Rs_mfont + 4, "mfont4", "fontname"),
 # endif
-# if NFONTS > 5
-    RSTRG(rs.mfont[5], "mfont5", "fontname"),
+# if MAX_NFONTS > 5
+    RSTRG(Rs_mfont + 5, "mfont5", "fontname"),
 # endif
-# if NFONTS > 6
-    RSTRG(rs.mfont[6], "mfont6", "fontname"),
+# if MAX_NFONTS > 6
+    RSTRG(Rs_mfont + 6, "mfont6", "fontname"),
 # endif
-# if NFONTS > 7
-    RSTRG(rs.mfont[7], "mfont7", "fontname"),
+# if MAX_NFONTS > 7
+    RSTRG(Rs_mfont + 7, "mfont7", "fontname"),
 # endif
 #endif				/* MULTICHAR_SET */
 #ifdef MULTICHAR_SET
-    STRG(rs.multichar_encoding, "multichar_encoding", "km", "mode",
-	 "multiple-character font encoding; mode = eucj | sjis | big5 | gb | kr | noenc"),
+    STRG(Rs_multichar_encoding, "multichar_encoding", "km", "mode",
+	 "multichar encoding; mode = eucj|sjis|big5|gb|kr|noenc"),
 #endif				/* MULTICHAR_SET */
 #ifdef USE_XIM
-    STRG(rs.preeditType, "preeditType", "pt", "style",
-	 "input style of input method; style = OverTheSpot | OffTheSpot | Root"),
-    STRG(rs.inputMethod, "inputMethod", "im", "name", "name of input method"),
+    STRG(Rs_inputMethod, "inputMethod", "im", "name", "name of input method"),
+    STRG(Rs_preeditType, "preeditType", "pt", "style",
+	 "input style: style = OverTheSpot|OffTheSpot|Root"),
 #endif				/* USE_XIM */
 #ifdef GREEK_SUPPORT
-    STRG(rs.greek_keyboard, "greek_keyboard", "grk", "mode",
+    STRG(Rs_greek_keyboard, "greek_keyboard", "grk", "mode",
 	 "greek keyboard mapping; mode = iso | ibm"),
 #endif
-    STRG(rs.name, NULL, "name", "string",
+    STRG(Rs_name, NULL, "name", "string",
 	 "client instance, icon, and title strings"),
-    STRG(rs.title, "title", "title", "string", "title name for window"),
-    STRG(rs.title, NULL, "T", NULL, NULL),	/* short form */
-    STRG(rs.iconName, "iconName", "n", "string", "icon name for window"),
+    STRG(Rs_title, "title", "title", "string", "title name for window"),
+    STRG(Rs_title, NULL, "T", NULL, NULL),	/* short form */
+    STRG(Rs_iconName, "iconName", "n", "string", "icon name for window"),
 #ifndef NO_CURSORCOLOR
-    STRG(rs.color[Color_cursor], "cursorColor", "cr", "color", "cursor color"),
+    STRG(Rs_color + Color_cursor, "cursorColor", "cr", "color", "cursor color"),
 /* command-line option = resource name */
-    RSTRG(rs.color[Color_cursor2], "cursorColor2", "color"),
+    RSTRG(Rs_color + Color_cursor2, "cursorColor2", "color"),
 #endif				/* NO_CURSORCOLOR */
-    STRG(rs.color[Color_pointer], "pointerColor", "pr", "color",
+    STRG(Rs_color + Color_pointer, "pointerColor", "pr", "color",
 	 "pointer color"),
-    STRG(rs.color[Color_border], "borderColor", "bd", "color",
+    STRG(Rs_color + Color_border, "borderColor", "bd", "color",
 	 "border color"),
-    STRG(rs.saveLines, "saveLines", "sl", "number",
+    STRG(Rs_saveLines, "saveLines", "sl", "number",
 	 "number of scrolled lines to save"),
 #ifndef NO_FRILLS
-    STRG(rs.ext_bwidth, "externalBorder", "w", "number",
+    STRG(Rs_ext_bwidth, "externalBorder", "w", "number",
 	 "external border in pixels"),
-    STRG(rs.ext_bwidth, NULL, "bw", NULL, NULL),
-    STRG(rs.ext_bwidth, NULL, "borderwidth", NULL, NULL),
-    STRG(rs.int_bwidth, "internalBorder", "b", "number",
+    STRG(Rs_ext_bwidth, NULL, "bw", NULL, NULL),
+    STRG(Rs_ext_bwidth, NULL, "borderwidth", NULL, NULL),
+    STRG(Rs_int_bwidth, "internalBorder", "b", "number",
 	 "internal border in pixels"),
 #endif
 #ifndef NO_BACKSPACE_KEY
-    RSTRG(rs.backspace_key, "backspacekey", "string"),
+    RSTRG(Rs_backspace_key, "backspacekey", "string"),
 #endif
 #ifndef NO_DELETE_KEY
-    RSTRG(rs.delete_key, "deletekey", "string"),
+    RSTRG(Rs_delete_key, "deletekey", "string"),
 #endif
-    RSTRG(rs.selectstyle, "selectstyle", "string"),
+    RSTRG(Rs_selectstyle, "selectstyle", "string"),
+    RSTRG(Rs_scrollstyle, "scrollstyle", "string"),
 #ifdef PRINTPIPE
-    RSTRG(rs.print_pipe, "print-pipe", "string"),
+    RSTRG(Rs_print_pipe, "print-pipe", "string"),
 #endif
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
-    RSTRG(rs.bigfont_key, "bigfont_key", "keysym"),
-    RSTRG(rs.smallfont_key, "smallfont_key", "keysym"),
+    RSTRG(Rs_bigfont_key, "bigfont_key", "keysym"),
+    RSTRG(Rs_smallfont_key, "smallfont_key", "keysym"),
 #endif
-    STRG(rs.modifier, "modifier", "mod", "modifier",
-	 "alt | meta | hyper | super | mod1 | ... | mod5"),
+    STRG(Rs_modifier, "modifier", "mod", "modifier",
+	 "meta modifier = alt|meta|hyper|super|mod1|...|mod5"),
+    INFO("xrm", "string", "X resource"),
 #ifdef CUTCHAR_RESOURCE
-    RSTRG(rs.cutchars, "cutchars", "string"),
+    RSTRG(Rs_cutchars, "cutchars", "string"),
 #endif				/* CUTCHAR_RESOURCE */
+    RSTRG(Rs_answerbackstring, "answerbackString", "string"),
     INFO("e", "command arg ...", "command to execute")
 };
 
@@ -275,108 +289,133 @@ static const struct {
 #undef BOOL
 /*}}} */
 
+static const char releasestring[] = "Rxvt v" VERSION " - released: " DATE "\n";
+static const char optionsstring[] = "Options: "
+#if defined(XPM_BACKGROUND)
+    "XPM,"
+#endif
+#if defined(TRANSPARENT)
+    "transparent,"
+#endif
+#if defined(UTMP_SUPPORT)
+    "utmp,"
+#endif
+#if defined(MENUBAR)
+    "menubar,"
+#endif
+#if !defined(NO_FRILLS)
+    "frills,"
+#endif
+#if defined(PREFER_24BIT)
+    "24bit,"
+#endif
+#if defined(USE_XIM)
+    "XIM,"
+#endif
+#if defined(MULTICHAR_SET)
+    "multichar_languages,"
+#endif
+    "scrollbars="
+#if !defined(HAVE_SCROLLBARS)
+    "NONE"
+#else
+# if defined(RXVT_SCROLLBAR)
+    "rxvt"
+#  if defined(NEXT_SCROLLBAR) || defined(XTERM_SCROLLBAR)
+    "+"
+#  endif
+# endif
+# if defined(NEXT_SCROLLBAR)
+    "NeXT"
+#  if defined(XTERM_SCROLLBAR)
+    "+"
+#  endif
+# endif
+# if defined(XTERM_SCROLLBAR)
+    "xterm"
+# endif
+#endif
+    ","
+#if defined(GREEK_SUPPORT)
+    "Greek,"
+#endif
+#if defined(RXVT_GRAPHICS)
+    "graphics,"
+#endif
+#if defined(NO_BACKSPACE_KEY)
+    "no_backspace,"
+#endif
+#if defined(NO_DELETE_KEY)
+    "no_delete,"
+#endif
+#if !defined(NO_STRINGS)
+    "strings,"
+#endif
+#if defined(NO_RESOURCES)
+    "NoResources"
+#else
+# if defined(USE_XGETDEFAULT)
+    "XGetDefaults"
+# else
+    ".Xdefaults"
+# endif
+#endif
+    "\nUsage: ";		/* Usage */
+
+#define INDENT 18
+
 /*{{{ usage: */
 /*----------------------------------------------------------------------*/
 /* EXTPROTO */
 void
-usage(int type)
+rxvt_usage(int type)
 {
-    int             i, col;
+    unsigned int    i, col;
 
-#define INDENT 30
-    fprintf(stderr, "\nUsage v%s : (", VERSION);
-#ifdef XPM_BACKGROUND
-# ifdef XPM_BUFFERING
-    fprintf(stderr, "XPM-buffer,");
-# else
-    fprintf(stderr, "XPM,");
-# endif
-#endif
-#ifdef UTMP_SUPPORT
-    fprintf(stderr, "utmp,");
-#endif
-#ifdef MENUBAR
-    fprintf(stderr, "menubar,");
-#endif
-#ifdef USE_XIM
-    fprintf(stderr, "XIM,");
-#endif
-#ifdef MULTICHAR_SET
-    fprintf(stderr, "multichar languages,");
-#endif
-#ifdef XTERM_SCROLLBAR
-    fprintf(stderr, "XTerm-scrollbar,");
-#endif
-#ifdef GREEK_SUPPORT
-    fprintf(stderr, "Greek,");
-#endif
-#ifdef RXVT_GRAPHICS
-    fprintf(stderr, "graphics,");
-#endif
-#ifdef NO_BACKSPACE_KEY
-    fprintf(stderr, "no backspace,");
-#endif
-#ifdef NO_DELETE_KEY
-    fprintf(stderr, "no delete,");
-#endif
-#ifdef NO_RESOURCES
-    fprintf(stderr, "NoResources");
-#else
-# ifdef USE_XGETDEFAULT
-    fprintf(stderr, "XGetDefaults");
-# else
-    fprintf(stderr, xnames[0]);
-# endif
-#endif
+    write(STDERR_FILENO, releasestring, sizeof(releasestring) - 1);
+    write(STDERR_FILENO, optionsstring, sizeof(optionsstring) - 1);
+    write(STDERR_FILENO, APL_NAME, sizeof(APL_NAME) - 1);
 
-    fprintf(stderr, ")\n%s", APL_NAME);
     switch (type) {
     case 0:			/* brief listing */
-	fprintf(stderr, " [-help]\n");
-	col = 3;
-	for (i = 0; i < optList_size(); i++) {
+	fprintf(stderr, " [-help] [--help]\n");
+	for (col = 1, i = 0; i < optList_size(); i++)
 	    if (optList[i].desc != NULL) {
-		int             len = 2;
+		int             len = 0;
 
 		if (!optList_isBool(i)) {
-		    len = optList_STRLEN(i);
+		    len = optList_strlen(i);
 		    if (len > 0)
 			len++;	/* account for space */
 		}
-		len += 4 + STRLEN(optList[i].opt);
-
+		len += 4 + STRLEN(optList[i].opt) + (optList_isBool(i) ? 2: 0);
 		col += len;
 		if (col > 79) {	/* assume regular width */
-		    fprintf(stderr, "\n");
-		    col = 3 + len;
+		    putc('\n', stderr);
+		    col = 1 + len;
 		}
-		fprintf(stderr, " [-");
-		if (optList_isBool(i))
-		    fprintf(stderr, "/+");
-		fprintf(stderr, "%s", optList[i].opt);
-		if (optList_STRLEN(i))
+		fprintf(stderr, " [-%s%s", (optList_isBool(i) ? "/+" : ""),
+			optList[i].opt);
+		if (optList_strlen(i))
 		    fprintf(stderr, " %s]", optList[i].arg);
 		else
 		    fprintf(stderr, "]");
 	    }
-	}
-	fprintf(stderr, "\n\n");
 	break;
 
     case 1:			/* full command-line listing */
 	fprintf(stderr, " [options] [-e command args]\n\n"
 		"where options include:\n");
-
 	for (i = 0; i < optList_size(); i++)
 	    if (optList[i].desc != NULL)
-		fprintf(stderr, "    %s%s %-*s%s%s\n",
-			(optList_isBool(i) ? "-/+" : "-"),
-			optList[i].opt, (INDENT - STRLEN(optList[i].opt)
-					 + (optList_isBool(i) ? 0 : 2)),
+		fprintf(stderr, "  %s%s %-*s%s%s\n",
+			(optList_isBool(i) ? "-/+" : "-"), optList[i].opt,
+			(INDENT - STRLEN(optList[i].opt)
+			 + (optList_isBool(i) ? 0 : 2)),
 			(optList[i].arg ? optList[i].arg : ""),
 			(optList_isBool(i) ? "turn on/off " : ""),
 			optList[i].desc);
-	fprintf(stderr, "\n    --help to list long-options\n\n");
+	fprintf(stderr, "\n  --help to list long-options");
 	break;
 
     case 2:			/* full resource listing */
@@ -386,18 +425,19 @@ usage(int type)
 
 	for (i = 0; i < optList_size(); i++)
 	    if (optList[i].kw != NULL)
-		fprintf(stderr, "    %s: %*s\n",
+		fprintf(stderr, "  %s: %*s%s\n",
 			optList[i].kw,
-			(INDENT - STRLEN(optList[i].kw)),
+			(INDENT - STRLEN(optList[i].kw)), "", /* XXX */
 			(optList_isBool(i) ? "boolean" : optList[i].arg));
-
 #ifdef KEYSYM_RESOURCE
-	fprintf(stderr, "    " "keysym.sym" ": %*s\n",
-		(INDENT - sizeof("keysym.sym") + 1), "keysym");
+	fprintf(stderr, "  " "keysym.sym" ": %*s%s\n",
+		(INDENT - sizeof("keysym.sym") + 1), "", /* XXX */
+		"keysym");
 #endif
-	fprintf(stderr, "\n    -help to list options\n\n");
+	fprintf(stderr, "\n  -help to list options");
 	break;
     }
+    fprintf(stderr, "\n\n");
     exit(EXIT_FAILURE);
     /* NOTREACHED */
 }
@@ -407,13 +447,13 @@ usage(int type)
 /*{{{ get command-line options before getting resources */
 /* EXTPROTO */
 void
-get_options(int argc, const char *const *argv)
+rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 {
     int             i, bad_option = 0;
     static const char On[3] = "ON", Off[4] = "OFF";
 
     for (i = 1; i < argc; i++) {
-	int             entry, longopt = 0;
+	unsigned int    entry, longopt = 0;
 	const char     *flag, *opt;
 
 	opt = argv[i];
@@ -430,14 +470,14 @@ get_options(int argc, const char *const *argv)
 		longopt = *opt++;	/* long option */
 	} else {
 	    bad_option = 1;
-	    print_error("bad option \"%s\"", opt);
+	    rxvt_print_error("bad option \"%s\"", opt);
 	    continue;
 	}
 
 	if (!STRCMP(opt, "help"))
-	    usage(longopt ? 2 : 1);
+	    rxvt_usage(longopt ? 2 : 1);
 	if (!STRCMP(opt, "h"))
-	    usage(0);
+	    rxvt_usage(0);
 
 	/* feature: always try to match long-options */
 	for (entry = 0; entry < optList_size(); entry++)
@@ -449,7 +489,7 @@ get_options(int argc, const char *const *argv)
 	if (entry < optList_size()) {
 	    if (optList_isReverse(entry))
 		flag = flag == On ? Off : On;
-	    if (optList_STRLEN(entry)) {	/* string value */
+	    if (optList_strlen(entry)) {	/* string value */
 		const char     *str = argv[++i];
 
 #ifdef DEBUG_RESOURCES
@@ -457,11 +497,11 @@ get_options(int argc, const char *const *argv)
 			optList[entry].opt ? optList[entry].opt : "nil",
 			optList[entry].kw ? optList[entry].kw : "nil");
 #endif
-		if (flag == On && str && optList[entry].dp) {
+		if (flag == On && str && (optList[entry].doff != -1)) {
 #ifdef DEBUG_RESOURCES
 		    fprintf(stderr, "\"%s\"\n", str);
 #endif
-		    *(optList[entry].dp) = str;
+		    r->h->rs[optList[entry].doff] = str;
 		    /*
 		     * special cases are handled in main.c:main() to allow
 		     * X resources to set these values before we settle for
@@ -478,21 +518,21 @@ get_options(int argc, const char *const *argv)
 			optList[entry].opt, optList[entry].kw, flag);
 #endif
 		if (flag == On)
-		    Options |= (optList[entry].flag);
+		    r->Options |= (optList[entry].flag);
 		else
-		    Options &= ~(optList[entry].flag);
+		    r->Options &= ~(optList[entry].flag);
 
-		if (optList[entry].dp)
-		    *(optList[entry].dp) = flag;
+		if (optList[entry].doff != -1)
+		    r->h->rs[optList[entry].doff] = flag;
 	    }
 	} else
 #ifdef KEYSYM_RESOURCE
 	    /* if (!STRNCMP(opt, "keysym.", sizeof("keysym.") - 1)) */
-	if (Str_match(opt, "keysym.")) {
+	if (rxvt_Str_match(opt, "keysym.")) {
 	    const char     *str = argv[++i];
 
 	    if (str != NULL)
-		parse_keysym(opt + sizeof("keysym.") - 1, str);
+		rxvt_parse_keysym(r, opt + sizeof("keysym.") - 1, str);
 	} else
 #endif
 	{
@@ -509,19 +549,19 @@ get_options(int argc, const char *const *argv)
 	    } else if (!STRCMP(opt, "7") || !STRCMP(opt, "8")
 #ifdef GREEK_SUPPORT
 		       /* obsolete 12 May 1996 (v2.17) */
-		       || !Str_match(opt, "grk")
+		       || !rxvt_Str_match(opt, "grk")
 #endif
 		)
 		msg = "obsolete";
 	    else
 		bad_option = 1;
 
-	    print_error("%s option \"%s\"", msg, --opt);
+	    rxvt_print_error("%s option \"%s\"", msg, --opt);
 	}
     }
 
     if (bad_option)
-	usage(0);
+	rxvt_usage(0);
 }
 
 /*}}} */
@@ -539,14 +579,15 @@ get_options(int argc, const char *const *argv)
 /* ARGSUSED */
 /* INTPROTO */
 Bool
-define_key(XrmDatabase *database, XrmBindingList bindings, XrmQuarkList quarks, XrmRepresentation *type, XrmValue *value, XPointer closure)
+rxvt_define_key(XrmDatabase *database __attribute__((unused)), XrmBindingList bindings __attribute__((unused)), XrmQuarkList quarks, XrmRepresentation *type __attribute__((unused)), XrmValue *value, XPointer closure __attribute__((unused)))
 {
     int             last;
+    rxvt_t         *r = rxvt_get_r();
 
     for (last = 0; quarks[last] != NULLQUARK; last++)	/* look for last quark in list */
 	;
     last--;
-    parse_keysym(XrmQuarkToString(quarks[last]), (char *)value->addr);
+    rxvt_parse_keysym(r, XrmQuarkToString(quarks[last]), (char *)value->addr);
     return False;
 }
 
@@ -561,14 +602,14 @@ define_key(XrmDatabase *database, XrmBindingList bindings, XrmQuarkList quarks,
 #define NEWARGLIM	500	/* `reasonable' size */
 /* INTPROTO */
 int
-parse_keysym(const char *str, const char *arg)
+rxvt_parse_keysym(rxvt_t *r, const char *str, const char *arg)
 {
     int             n, sym;
     char           *key_string, *newarg = NULL;
     char            newargstr[NEWARGLIM];
 
     if (arg == NULL) {
-	if ((n = Str_match(str, "keysym.")) == 0)
+	if ((n = rxvt_Str_match(str, "keysym.")) == 0)
 	    return 0;
 	str += n;		/* skip `keysym.' */
     }
@@ -606,7 +647,7 @@ parse_keysym(const char *str, const char *arg)
     if (sym < 0xFF00 || sym > 0xFFFF)	/* we only do extended keys */
 	return -1;
     sym &= 0xFF;
-    if (KeySym_map[sym] != NULL)	/* already set ? */
+    if (r->h->Keysym_map[sym] != NULL)	/* already set ? */
 	return -1;
 
     if (newarg == NULL) {
@@ -614,15 +655,15 @@ parse_keysym(const char *str, const char *arg)
 	newargstr[NEWARGLIM - 1] = '\0';
 	newarg = newargstr;
     }
-    Str_trim(newarg);
-    if (*newarg == '\0' || (n = Str_escaped(newarg)) == 0)
+    rxvt_Str_trim(newarg);
+    if (*newarg == '\0' || (n = rxvt_Str_escaped(newarg)) == 0)
 	return -1;
     MIN_IT(n, 255);
-    key_string = MALLOC((n + 1) * sizeof(char));
+    key_string = malloc((n + 1) * sizeof(char));
 
     key_string[0] = n;
     STRNCPY(key_string + 1, newarg, n);
-    KeySym_map[sym] = (unsigned char *)key_string;
+    r->h->Keysym_map[sym] = (unsigned char *)key_string;
 
     return 1;
 }
@@ -630,13 +671,13 @@ parse_keysym(const char *str, const char *arg)
 # endif				/* KEYSYM_RESOURCE */
 
 # ifndef USE_XGETDEFAULT
-/*{{{ get_xdefaults() */
+/*{{{ rxvt_get_xdefaults() */
 /*
  * the matching algorithm used for memory-save fake resources
  */
 /* INTPROTO */
 void
-get_xdefaults(FILE *stream, const char *name)
+rxvt_get_xdefaults(rxvt_t *r, FILE *stream, const char *name)
 {
     unsigned int    len;
     char           *str, buffer[256];
@@ -656,7 +697,7 @@ get_xdefaults(FILE *stream, const char *name)
 	str += (len + 1);	/* skip `name*' or `name.' */
 
 # ifdef KEYSYM_RESOURCE
-	if (!parse_keysym(str, NULL))
+	if (!rxvt_parse_keysym(r, str, NULL))
 # endif				/* KEYSYM_RESOURCE */
 	    for (entry = 0; entry < optList_size(); entry++) {
 		const char     *kw = optList[entry].kw;
@@ -664,18 +705,18 @@ get_xdefaults(FILE *stream, const char *name)
 		if (kw == NULL)
 		    continue;
 		n = STRLEN(kw);
-		if (str[n] == ':' && Str_match(str, kw)) {
+		if (str[n] == ':' && rxvt_Str_match(str, kw)) {
 		    /* skip `keyword:' */
 		    str += (n + 1);
-		    Str_trim(str);
+		    rxvt_Str_trim(str);
 		    n = STRLEN(str);
-		    if (n && *(optList[entry].dp) == NULL) {
+		    if (n && r->h->rs[optList[entry].doff] == NULL) {
 			/* not already set */
 			int             s;
-			char           *p = MALLOC((n + 1) * sizeof(char));
+			char           *p = malloc((n + 1) * sizeof(char));
 
 			STRCPY(p, str);
-			*(optList[entry].dp) = p;
+			r->h->rs[optList[entry].doff] = p;
 			if (optList_isBool(entry)) {
 			    s = STRCASECMP(str, "TRUE") == 0
 				|| STRCASECMP(str, "YES") == 0
@@ -684,9 +725,9 @@ get_xdefaults(FILE *stream, const char *name)
 			    if (optList_isReverse(entry))
 				s = !s;
 			    if (s)
-				Options |= (optList[entry].flag);
+				r->Options |= (optList[entry].flag);
 			    else
-				Options &= ~(optList[entry].flag);
+				r->Options &= ~(optList[entry].flag);
 			}
 		    }
 		    break;
@@ -707,9 +748,24 @@ get_xdefaults(FILE *stream, const char *name)
 /* ARGSUSED */
 /* EXTPROTO */
 void
-extract_resources(Display *display, const char *name)
+rxvt_extract_resources(rxvt_t *r, Display *display __attribute__((unused)), const char *name)
 {
 #ifndef NO_RESOURCES
+
+# if defined XAPPLOADDIR
+#  if defined(HAVE_XSETLOCALE) || defined(HAVE_SETLOCALE)
+    /* Compute the path of the possibly available localized Rxvt file */ 
+    char           *localepath = NULL;
+
+    if (r->h->locale != NULL) {	/* XXX: must limit length of string */
+	localepath = malloc(256); 
+	sprintf(localepath, XAPPLOADDIRLOCALE "/" APL_SUBCLASS, r->h->locale);
+    }
+
+    {
+#  endif
+# endif
+
 # ifdef USE_XGETDEFAULT
 /*
  * get resources using the X library function
@@ -733,7 +789,7 @@ extract_resources(Display *display, const char *name)
     if (displayResource != NULL)
 	database = XrmGetStringDatabase(displayResource);
 
-#ifdef HAVE_EXTRA_XRESOURCE_FILES
+#   ifdef HAVE_EXTRA_XRESOURCE_FILES
 /* Add in ~/.Xdefaults or ~/.Xresources */
     {
 	char           *ptr;
@@ -746,13 +802,13 @@ extract_resources(Display *display, const char *name)
 		    ptr, xnames[i]);
 	    if ((rdb1 = XrmGetFileDatabase(fname)) != NULL) {
 		XrmMergeDatabases(rdb1, &database);
-# ifndef HAVE_BOTH_XRESOURCE_FILES
+#    ifndef HAVE_BOTH_XRESOURCE_FILES
 		break;
-# endif
+#    endif
 	    }
 	}
     }
-#endif
+#   endif
 
 /* Add in XENVIRONMENT file */
 
@@ -761,9 +817,12 @@ extract_resources(Display *display, const char *name)
 	XrmMergeDatabases(rdb1, &database);
 
 /* Add in Rxvt file */
-
-    if ((rdb1 = XrmGetFileDatabase(XAPPLOADDIR "/" APL_SUBCLASS)) != NULL)
-	XrmMergeDatabases(rdb1, &database);
+#   if defined(HAVE_XSETLOCALE) || defined(HAVE_SETLOCALE)
+    if (localepath == NULL || (rdb1 = XrmGetFileDatabase(localepath)) == NULL)
+#   endif
+    rdb1 = XrmGetFileDatabase(XAPPLOADDIR "/" APL_SUBCLASS);
+    if (rdb1 != NULL)
+        XrmMergeDatabases(rdb1, &database);
 
     XrmSetDatabase(display, database);
 #  endif
@@ -776,12 +835,12 @@ extract_resources(Display *display, const char *name)
 	char           *p;
 	const char     *kw = optList[entry].kw;
 
-	if (kw == NULL || *(optList[entry].dp) != NULL)
+	if (kw == NULL || r->h->rs[optList[entry].doff] != NULL)
 	    continue;		/* previously set */
 	if ((p = XGetDefault(display, name, kw)) != NULL
 	    || (p = XGetDefault(display, APL_SUBCLASS, kw)) != NULL
 	    || (p = XGetDefault(display, APL_CLASS, kw)) != NULL) {
-	    *optList[entry].dp = p;
+	    r->h->rs[optList[entry].doff] = p;
 
 	    if (optList_isBool(entry)) {
 		s = STRCASECMP(p, "TRUE") == 0
@@ -791,9 +850,9 @@ extract_resources(Display *display, const char *name)
 		if (optList_isReverse(entry))
 		    s = !s;
 		if (s)
-		    Options |= (optList[entry].flag);
+		    r->Options |= (optList[entry].flag);
 		else
-		    Options &= ~(optList[entry].flag);
+		    r->Options &= ~(optList[entry].flag);
 	    }
 	}
     }
@@ -809,14 +868,16 @@ extract_resources(Display *display, const char *name)
     class_prefix[0] = XrmStringToName(APL_SUBCLASS);
     class_prefix[1] = XrmStringToName("Keysym");
     class_prefix[2] = NULLQUARK;
+/* XXX: Need to check sizeof(rxvt_t) == sizeof(XPointer) */
     XrmEnumerateDatabase(XrmGetDatabase(display), name_prefix, class_prefix,
-			 XrmEnumOneLevel, define_key, NULL);
+			 XrmEnumOneLevel, rxvt_define_key, NULL);
     name_prefix[0] = XrmStringToName(APL_CLASS);
     name_prefix[1] = XrmStringToName("keysym");
     class_prefix[0] = XrmStringToName(APL_CLASS);
     class_prefix[1] = XrmStringToName("Keysym");
+/* XXX: Need to check sizeof(rxvt_t) == sizeof(XPointer) */
     XrmEnumerateDatabase(XrmGetDatabase(display), name_prefix, class_prefix,
-			 XrmEnumOneLevel, define_key, NULL);
+			 XrmEnumOneLevel, rxvt_define_key, NULL);
 #   endif
 #  endif
 
@@ -826,7 +887,7 @@ extract_resources(Display *display, const char *name)
     char           *home;
 
     if ((home = getenv("HOME")) != NULL) {
-	int             i, len = STRLEN(home) + 2;
+	unsigned int    i, len = STRLEN(home) + 2;
 	char           *f = NULL;
 
 	for (i = 0; i < (sizeof(xnames) / sizeof(xnames[0])); i++) {
@@ -837,7 +898,7 @@ extract_resources(Display *display, const char *name)
 	    if ((fd = fopen(f, "r")) != NULL)
 		break;
 	}
-	FREE(f);
+	free(f);
     }
 /*
  * The normal order to match resources is the following:
@@ -861,25 +922,39 @@ extract_resources(Display *display, const char *name)
  * "XTerm" and "Rxvt" as class names.
  */
 
-    get_xdefaults(fd, name);
-    get_xdefaults(fd, APL_SUBCLASS);
+    rxvt_get_xdefaults(r, fd, name);
+    rxvt_get_xdefaults(r, fd, APL_SUBCLASS);
 
 #  if defined(XAPPLOADDIR) && defined(USE_XAPPLOADDIR)
     {
-	FILE           *ad = fopen(XAPPLOADDIR "/" APL_SUBCLASS, "r");
+	FILE           *ad = NULL;
 
+#   if defined(HAVE_XSETLOCALE) || defined(HAVE_SETLOCALE)
+	if (localepath == NULL || (ad = fopen(localepath, "r")) == NULL)
+#   endif
+	ad = fopen(XAPPLOADDIR "/" APL_SUBCLASS, "r");
 	if (ad != NULL) {
-	    get_xdefaults(ad, "");
+	    rxvt_get_xdefaults(r, ad, "");
 	    fclose(ad);
 	}
     }
 #  endif			/* XAPPLOADDIR */
 
-    get_xdefaults(fd, APL_CLASS);
-    get_xdefaults(fd, "");	/* partial match */
+    rxvt_get_xdefaults(r, fd, APL_CLASS);
+    rxvt_get_xdefaults(r, fd, "");	/* partial match */
     if (fd != NULL)
 	fclose(fd);
 # endif				/* USE_XGETDEFAULT */
+
+# if defined XAPPLOADDIR
+#  if defined(HAVE_XSETLOCALE) || defined(HAVE_SETLOCALE)
+    }
+
+    /* Free the path of the possibly available localized Rxvt file */ 
+    free(localepath);
+#  endif
+# endif
+
 #endif				/* NO_RESOURCES */
 
 /*
@@ -887,15 +962,15 @@ extract_resources(Display *display, const char *name)
  * options and command-line long options
  */
 #ifdef MULTICHAR_SET
-    set_multichar_encoding(rs.multichar_encoding);
+    rxvt_set_multichar_encoding(r, r->h->rs[Rs_multichar_encoding]);
 #endif
 #ifdef GREEK_SUPPORT
 /* this could be a function in grkelot.c */
 /* void set_greek_keyboard (const char * str); */
-    if (rs.greek_keyboard) {
-	if (!STRCMP(rs.greek_keyboard, "iso"))
+    if (r->h->rs[Rs_greek_keyboard]) {
+	if (!STRCMP(r->h->rs[Rs_greek_keyboard], "iso"))
 	    greek_setmode(GREEK_ELOT928);	/* former -grk9 */
-	else if (!STRCMP(rs.greek_keyboard, "ibm"))
+	else if (!STRCMP(r->h->rs[Rs_greek_keyboard], "ibm"))
 	    greek_setmode(GREEK_IBM437);	/* former -grk4 */
     }
 #endif				/* GREEK_SUPPORT */
@@ -904,12 +979,12 @@ extract_resources(Display *display, const char *name)
     {
 	KeySym          sym;
 
-	if (rs.bigfont_key
-	    && ((sym = XStringToKeysym(rs.bigfont_key)) != 0))
-	    ks_bigfont = sym;
-	if (rs.smallfont_key
-	    && ((sym = XStringToKeysym(rs.smallfont_key)) != 0))
-	    ks_smallfont = sym;
+	if (r->h->rs[Rs_bigfont_key]
+	    && ((sym = XStringToKeysym(r->h->rs[Rs_bigfont_key])) != 0))
+	    r->h->ks_bigfont = sym;
+	if (r->h->rs[Rs_smallfont_key]
+	    && ((sym = XStringToKeysym(r->h->rs[Rs_smallfont_key])) != 0))
+	    r->h->ks_smallfont = sym;
     }
 #endif
 }
